function [EigVal,Pert] = EigValProbSolver(EVP,varargin)
% This function is the computational backend which solves the general
% eigenvalue stability problem.
%
% 'Modes' optional argument is not tested.
% No set sign for perturbation profiles - May be negative.
%
% Inputs: EVP - The eigenvalue problem structure
%                Required fields:
%                   IPs        - Internal point indexes
%                   BPs        - Boundary point indexes
%                   OpBPs      - Boundary point operator
%                   OpMain     - Main eigenvalue operator
%                   OpGrow     - Growth eigenvalue operator
%                   OpRayl     - Rayleigh eigenvalue operator
%         
% (Opt.)  ProbType - Problem type flag:
%                      1 = Solve for critical Rayleigh number
%                      0 = Solve for growth rate     
% (Opt.)  RaylGrow - Specified Rayleigh number/growth rate
%                      (Must be supplied if 'ProbType' is supplied)
% (Opt.)  Modes    - Number of modes to return
%
% Outputs: EigVal - The eigenvalue(s) requested
%          Pert   - The perturbation profile(s) requested
% 
% CoreFunctions Version 1.0
% (04/02/15)
    
    % Check correct argument range
    narginchk(1,4)

    % Process optional 'ProbType' and 'RaylGrow' arguments
    if nargin > 1
        
        % Set 'ProbType' and 'RaylGrow'
        ProbType = varargin{1};
        RaylGrow = varargin{2};
    
    % Set defaults
    else
        ProbType = 1;
        RaylGrow = 0;
    end
    
    % Process optional 'Modes' arguement
    if nargin > 3
        Modes = varargin{3};
    
    % Set default
    else
        Modes = 1;
    end

    % Specific eigenvalue operators
    if ProbType
        OpMain = EVP.OpMain - EVP.OpGrow*RaylGrow;
        OpScnd = EVP.OpRayl;
    else
        OpMain = EVP.OpMain - EVP.OpRayl*RaylGrow;
        OpScnd = EVP.OpGrow;
    end
    
    % Apply boundary conditions
    OpMain = OpMain(EVP.IPs,EVP.IPs) + OpMain(EVP.IPs,EVP.BPs)*EVP.OpBPs;
    OpScnd = OpScnd(EVP.IPs,EVP.IPs) + OpScnd(EVP.IPs,EVP.BPs)*EVP.OpBPs;
    
    % Call routine
    [EigVec,EigVal] = eig(OpMain,OpScnd);
    EigVal          = diag(EigVal);
    
    % Sort Rayleigh number results
    if ProbType
        EigVec     = EigVec(:,isfinite(real(EigVal)) & (EigVal>0));
        EigVal     = EigVal(isfinite(real(EigVal)) & (EigVal>0));
        [~,tmpInd] = sort(EigVal,'ascend');
        EigVal     = EigVal(tmpInd(Modes));
        EigVec     = EigVec(:,tmpInd(Modes));
     
    % Sort growth rate results
    else
        EigVec     = EigVec(:,isfinite(real(EigVal)));
        EigVal     = EigVal(isfinite(real(EigVal)));
        [~,tmpInd] = sort(EigVal,'descend');
        EigVal     = EigVal(tmpInd(Modes));
        EigVec     = EigVec(:,tmpInd(Modes));
    end
    
    % Generate, normalise and assign perturbation profiles
    BPs = EVP.OpBPs*EigVec;
    Pert(EVP.IPs,:) = EigVec;
    Pert(EVP.BPs,:) = BPs;
    Pert = Pert/sqrt(Pert'*Pert);
    if Pert( abs(Pert) == max(abs(Pert)) ) < 0
        Pert = -Pert;
    end
end
