function [EVP,Indx] = InitialiseEVP(Model,Phys,Num)
% This script initialises the EigenValue Problem (EVP) structure.
%
% Inputs: Model - Model structure
%                  Required fields:
%                    NPerts    - Number of perturbations
%                    NFields   - Number of fields (total)
%                    Fields    - Number of fields (for each perturbation)
%                    FieldNums - Field numbers structure
%                    BCInd     - Boundary condition number structure
%         
%         Phys - Physical problem structure
%                  Required fields:
%                    BCs   - Boundary condition structure
%                    Bi    - Biot number
%
%         Num   - Numerical structure
%                  Required fields:
%                    Nz   - Number of grid points
%                    OpD1 - First derivative operator
%                    OpD2 - Second derivative operator
% 
% Outputs: EVP - The eigenvalue problem structure
%                  Contains fields:
%                    BPs        - Boundary point indicies
%                    IPs        - Internal point indicies
%                    EVP.OpBPs  - Boundary point operator
%
%          Indx - Perturbation field indicies
% 
% CoreFunctions Version 2.0
% (14/05/15)
    
    % Prepare EVP
    Indx = [];
    EVP.BPs = [];
    EVP.IPs = [];
    
    % Loop over variables
    OpBC = zeros(2*Model.NFields,Model.NFields*(Num.Nz+1));
    for iV = 1:Model.NPerts
        
        % Loop over fields
        for iF = 1:Model.Fields(iV)
            iI = Model.FieldNums{iV}(iF);
            
            % Add indicies
            Indx(:,iI) = (1:(Num.Nz+1))'+(Num.Nz+1)*(iI-1);     %#ok<AGROW>
            EVP.BPs = [EVP.BPs;Indx([1:Model.BCNums{iV}(iF),end+1+(-Model.BCNums{iV}(iF):-1)],iI)];
            EVP.IPs = [EVP.IPs;Indx((Model.BCNums{iV}(iF)+1):(end-Model.BCNums{iV}(iF)),iI)];
        end
        
        % Loop over boundary conditions
        for iF = Model.BCInd{iV}
            
            % Upper boundary
            if strcmpi(Phys.BCs{iF}(1),'U')
                BoundInd = [true,false(1,Num.Nz)];
                
            % Lower boundary
            else
                BoundInd = [false(1,Num.Nz),true];
            end
            
            % Set boundary type
            switch upper(Phys.BCs{iF}(2:3))
                
                % Zero value boundary
                case 'ZV'
                    OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = BoundInd;
                
                % Zero gradient boundary
                case 'ZG'
                    OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Num.OpD1(BoundInd,:);
                    
                % Zero second derivative boundary
                case 'Z2'
                    if Model.Fields(iV) == 1
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Num.OpD2(BoundInd,:);
                    else
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(2))) = BoundInd;
                    end
                    
                % Zero third derivative boundary
                case 'Z3'
                    if Model.Fields(iV) == 1
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Num.OpD1(BoundInd,:)*Num.OpD2;
                    else
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(2))) = Num.OpD1(BoundInd,:);
                    end
                    
                % Robin boundary (zeroth order)
                case 'R0'
                    if Phys.Bi >= 1
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = BoundInd - 1/Phys.Bi * Num.OpD1(BoundInd,:);
                    else
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Phys.Bi*BoundInd - Num.OpD1(BoundInd,:);
                    end
                    
                % Robin boundary (first order)
                case 'R1'
                    if Phys.Bi >= 1
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Num.OpD1(BoundInd,:) - 1/Phys.Bi * Num.OpD2(BoundInd,:);
                    else
                        OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Phys.Bi*Num.OpD1(BoundInd,:) - Num.OpD2(BoundInd,:);
                    end
                    
                % Robin boundary (second order)    
                case 'R2'
                    if Phys.Bi >= 1
                        
                        if Model.Fields(iV) == 1
                            OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Num.OpD2(BoundInd,:) - 1/Phys.Bi * Num.OpD1(BoundInd,:)*Num.OpD2;
                        else
                            OpBC(iF,Indx(:,Model.FieldNums{iV}(2))) = BoundInd - 1/Phys.Bi * Num.OpD1(BoundInd,:);
                        end
                        
                    else
                        
                        if Model.Fields(iV) == 1
                            OpBC(iF,Indx(:,Model.FieldNums{iV}(1))) = Phys.Bi*Num.OpD2(BoundInd,:) - Num.OpD1(BoundInd,:)*Num.OpD2;
                        else
                            OpBC(iF,Indx(:,Model.FieldNums{iV}(2))) = Phys.Bi*BoundInd - Num.OpD1(BoundInd,:);
                        end
                    end
            end
        end
    end
    
    % Create boundary point operator
    EVP.OpBPs = -OpBC(:,EVP.BPs)\OpBC(:,EVP.IPs);
end
