function Mod = InitialiseModel(ModSpec)
% This function initialises the model structure for a given model
% specifier.
% 
% Input: ModSpec - Model specifier code
%                    Medium specifier: (first letter)
%                      'P' = Porous medium
%                      'L' = Liquid medium
%                      'M' = Mushy medium
%                    Method specifier: (second letter)
%                      'Q' = QSSA
%                      'P' = Propagation
%                      'E' = Energy analysis
%                    Momentum specifier: (third letter)
%                      '1' = Single momentum equation
%                      '2' = Split  momentum equation
%
% Output: Mod - Model structure
%                 Contains:
%                   ModSpec   - Model specifier code
%                   Medium    - Medium binary flag
%                   Stability - Stability method binary flag
%                   Momentum  - Split momentum binary flag
%                   NPerts    - Number of perturbations
%                   Perts     - Perturbation names
%                   NFields   - Number of fields (total)
%                   Fields    - Number of fields (for each perturbation)
%                   FieldNums - Field numbers structure
%                   BCInd     - Boundary condition number structure
% 
% CoreFunctions Version 2.0
% (14/05/15)
    
    
    %%% Process model
    Mod.ModSpec   = ModSpec;
    
    % Process medium
    if     strcmpi(ModSpec(1),'P')
        Mod.Medium = 0;
        MomentumFields = 1;
    elseif strcmpi(ModSpec(1),'L')
        Mod.Medium = 1;
        MomentumFields = 2;
    elseif strcmpi(ModSpec(1),'M')
        Mod.Medium = 2;
        MomentumFields = 1;
    else
        error('Medium not recognised!')
    end
    
    % Process stability method
    if     strcmpi(ModSpec(2),'Q')
        Mod.Stability = 1;
    elseif strcmpi(ModSpec(2),'P')
        Mod.Stability = 2;
    elseif strcmpi(ModSpec(2),'E')
        Mod.Stability = 3;
    else
        error('Stability method not recognised!')
    end
    
    % Process momentum method
    if     strcmpi(ModSpec(3),'1')
        Mod.Momentum = 0;
    elseif strcmpi(ModSpec(3),'2')
        Mod.Momentum = 1;
    else
        error('Momentum method not recognised!')
    end
    
    
    %%% Generate model
    
    % Initialise structure
    Mod.NPerts    = 0;
    Mod.Perts     = {};
    Mod.NFields   = 0;
    Mod.Fields    = [];
    Mod.FieldNums = {};
    Mod.NBCs      = 0;
    Mod.BCNums    = {};
    Mod.BCInd     = {};
    
    % Initialise thermal perturbation
    AddPert('Thermal',1);
    
    % Initialise velocity perturbation
    AddPert('Velocity',MomentumFields);
    
    % Initialise lagrange perturbation
    if Mod.Stability == 3
        AddPert('Lagrange',MomentumFields)
    end
    
    
    %% AddPert() nested function
    function AddPert(Name,Fields)
    % This function adds a perturbation called 'Name' with 'Fields'
    % independent fields.
    %
    % CoreFunctions Version 2.0
    % (16/04/15)
        
        % Split momentum equation
        if Mod.Momentum
    
            % Add perturbation
            Mod.Perts     = [Mod.Perts,Name];
            Mod.Fields    = [Mod.Fields,Fields];
            Mod.FieldNums = [Mod.FieldNums, {Mod.NFields + (1:Fields)}];
            Mod.BCNums    = [Mod.BCNums, {ones(1,Fields)}];
            Mod.BCInd     = [Mod.BCInd, {Mod.NBCs + (1:(2*Fields))} ];
            
            % Update counts
            Mod.NPerts  = Mod.NPerts+1;
            Mod.NFields = Mod.NFields+Fields;
            Mod.NBCs    = Mod.NBCs+2*Fields;
            
        % Single momentum equation
        else
            
            % Add perturbation
            Mod.Perts     = [Mod.Perts,Name];
            Mod.Fields    = [Mod.Fields,1];
            Mod.FieldNums = [Mod.FieldNums, {Mod.NFields + 1}];
            Mod.BCNums    = [Mod.BCNums, {Fields}];
            Mod.BCInd     = [Mod.BCInd, {Mod.NBCs + (1:(2*Fields))} ];
            
            % Update counts
            Mod.NPerts  = Mod.NPerts+1;
            Mod.NFields = Mod.NFields+1;
            Mod.NBCs    = Mod.NBCs+2*Fields;
        end
    end
end

