function [OpMain,OpGrow,OpRayl] = MakeMainOperators(Model,Num,k)
% This function creates the eigenvalue operators used to calculate the
% stability properties.
%
% Inputs: Model - Model structure
%                  Required fields:
%                    ModSpec   - Model specifier code
%                    Medium    - Medium binary flag
%                    Stability - Stability method binary flag
%                    Momentum  - Split momentum binary flag
%                    NFields   - Number of fields (total)
%                    FieldNums - Field numbers structure
%         
%         Num - Numerical structure
%                Required fields:
%                  Nz          - Number of grid points
%                  Indx        - Perturbation field indicies
%                  OpLap       - Laplacian operator
%                  OpThermForc - Thermal forcing operator
%         
%         Ind - The field indexes as generated by InitialiseEVP()
%         k   - The investigation wavenumber
%
% Outputs: OpMain - The main operator
%          OpGrow - The growth rate operator
%          OpRayl - The Rayleigh number operator
%
% CoreFunctions Version 2.0
% (14/05/15)
    
    % Initialise operators
    OpMain = zeros( Model.NFields * (Num.Nz+1) );
    OpGrow = zeros( Model.NFields * (Num.Nz+1) );
    OpRayl = zeros( Model.NFields * (Num.Nz+1) );
    
    % Select equation order
    if  ~(Model.Medium==1) 
        Sign = -1;
    else
        Sign = 1;
    end
    
    % Add thermal equation
    OpMain(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{1}(1))) = Num.OpLap1;
    OpGrow(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{1}(1))) = Num.OpD0;
    OpMain(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{2}(1))) = 1.0 * k^2 * Num.OpThermForc;
    if Model.Stability == 2
        OpMain(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{1}(1))) = ...
            OpMain(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{1}(1))) + 0.5*diag(Num.Grid)*Num.OpD1;
    elseif Model.Stability == 3
        OpMain(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{2}(1))) =  0.5 * k^2   * Num.OpThermForc;
        OpRayl(Num.Indx(:,Model.FieldNums{1}(1)),Num.Indx(:,Model.FieldNums{3}(1))) = Sign * k^2/2 * Num.OpD0;
    end
    
    % Add momentum equation
    OpMain(Num.Indx(:,Model.FieldNums{2}( 1 )),Num.Indx(:,Model.FieldNums{2}(1))) =      Num.OpLapP;
    OpRayl(Num.Indx(:,Model.FieldNums{2}(end)),Num.Indx(:,Model.FieldNums{1}(1))) = Sign*Num.OpD0;
    if Model.Medium == 1
        if Model.Momentum
            OpMain(Num.Indx(:,Model.FieldNums{2}(1)),Num.Indx(:,Model.FieldNums{2}(2))) = -Num.OpD0;
            OpMain(Num.Indx(:,Model.FieldNums{2}(2)),Num.Indx(:,Model.FieldNums{2}(2))) =  Num.OpLap1;
        else
            OpMain(Num.Indx(:,Model.FieldNums{2}(1)),Num.Indx(:,Model.FieldNums{2}(1))) =  Num.OpLap2;
        end
    end
    
    % Add lagrange equation
    if Model.Stability == 3
        OpMain(Num.Indx(:,Model.FieldNums{3}( 1 )),Num.Indx(:,Model.FieldNums{3}(1))) = Num.OpLap1;
        OpMain(Num.Indx(:,Model.FieldNums{3}(end)),Num.Indx(:,Model.FieldNums{1}(1))) = Num.OpThermForc;
        if Model.Medium == 1
            if Model.Momentum
                OpMain(Num.Indx(:,Model.FieldNums{3}(1)),Num.Indx(:,Model.FieldNums{3}(2))) = -Num.OpD0;
                OpMain(Num.Indx(:,Model.FieldNums{3}(2)),Num.Indx(:,Model.FieldNums{3}(2))) =  Num.OpLap1;
            else
                OpMain(Num.Indx(:,Model.FieldNums{3}(1)),Num.Indx(:,Model.FieldNums{3}(1))) =  Num.OpLap2;
            end
        end
    end
end

