function [CritRayl,CritWave,OnsetRayl,varargout] = StabilitySnapshot(Model,Phys,Nz,WaveVec,varargin)
% This function calculates the stability characteristics for a range of
% wavenumbers (and optionally, Rayleigh numbers) with all other properties
% held constant.
%
% It currently only calculates the dominant mode and does not allow
% different boundary conditions.
%
% Inputs: Model - Model structure
%                  Required fields:
%                    ModSpec - Model specifier
%                    Medium    - Medium binary flag
%                    Stability - Stability method binary flag
%                    Momentum  - Split momentum binary flag
%                    NPerts    - Number of perturbations
%                    NFields   - Number of fields (total)
%                    Fields    - Number of fields (for each perturbation)
%                    FieldNums - Field numbers structure
%                    BCInd     - Boundary condition number structure
%         
%         Phys - The physical problem structure
%                 Required fields:
%                   Depth - Computational domain depth
%                   Temp  - Temperature profile
%                   BCs   - Boundary condition structure
%                   Bi    - Biot number
%         
%         Nz      - Number of grid points
%         WaveVec - Wavenumber vector
% (Opt.)  RaylVec - Rayleigh number vector
%
% Outputs: CritRayl  - Minimum critical Rayleigh number
%          CritWave  - Wavenumber at minimum Rayleigh number
%          OnsetRayl - Critical Rayleigh number vector
% (Opt.)   Growth    - Growth rate matrix
%                        (Only calculated if RaylVec provided)
% (Opt.)   Pert      - Perturbation vector
%
% CoreFunctions Version 2.0
% (14/05/15)
    
    % Create numerical structure
    Num.Nz          = Nz;
    Num.Grid        = ChebGrid(Num.Nz,Phys.Depth);
    Num.OpD0        = eye(Nz+1);
    Num.OpD1        = DiffMatrix(Num.Nz,Phys.Depth);
    Num.OpD2        = Num.OpD1 * Num.OpD1;
    
    % Forcing and properties
    Num.OpThermForc = diag( Num.OpD1 * Phys.Temp );
    if ~isfield(Phys,'LqdFrac')
        Phys.LqdFrac = ones(Num.Nz+1,1);
    end
    Phys.Perm = Phys.LqdFrac.^-3;
    if Model.Medium == 2
        [Phys.ThermEvol,Phys.LqFrcEvol] = EvolutionInversion(Phys,Num);
    end
    
    % Initialise EVP structure
    [EVP,Num.Indx] = InitialiseEVP(Model,Phys,Num);
    
    % Rayleigh number vector input
    CalcGrowthRates = false;
    if nargin > 4
        CalcGrowthRates = true;
        RaylVec = varargin{1};
    end
    
    % Prepare result containers
    OnsetRayl  = NaN(1,length(WaveVec));
    if CalcGrowthRates
        Growth = NaN(length(RaylVec),length(WaveVec));
    end
    
    % Loop over wavenumbers
    for iK = 1:length(WaveVec) 
        
        % Create operators
        Num.OpLap1 = (Num.OpD2 - WaveVec(iK)^2*eye(Num.Nz+1));
        Num.OpLapP = Num.OpD1*diag(Phys.Perm)*Num.OpD1 - diag(WaveVec(iK)^2*Phys.Perm);
        Num.OpLap2 = Num.OpLap1*Num.OpLap1;
        [EVP.OpMain,EVP.OpGrow,EVP.OpRayl] = MakeMainOperators(Model,Num,WaveVec(iK));
        
        % Calculate growth rates
        if CalcGrowthRates
            Growth(:,iK) = GetGrowthRates(EVP,RaylVec);
        end
        
        % Find critical Rayleigh number
        OnsetRayl(iK) = EigValProbSolver(EVP);
    end
    
    % Calculate stability minimum
    [CritWave,CritRayl] = FindStationaryPoint(WaveVec,OnsetRayl(1,:),0);
    
    % Set growth rate output
    varargout = {};
    if CalcGrowthRates
        varargout = Growth;
    end
    
    % Check perturbations output
    if (nargout > 4) || ( ~CalcGrowthRates && nargout > 3  )
        
        %%%
        
        % Create numerical structure
        Num2.Nz          = Nz;
        Num2.Grid        = ChebGrid(Num2.Nz,Phys.Depth);
        Num2.OpD0        = eye(Nz+1);
        Num2.OpD1        = DiffMatrix(Num2.Nz,Phys.Depth);
        Num2.OpD2        = Num2.OpD1 * Num2.OpD1;
        
        % Forcing and properties
        Num2.OpThermForc = diag( Num2.OpD1 * Phys.Temp );
        if ~isfield(Phys,'LqdFrac')
            Phys.LqdFrac = ones(Num2.Nz+1,1);
        end
        Phys.Perm = Phys.LqdFrac.^-3;
        if Model.Medium == 2
            [Phys.ThermEvol,Phys.LqFrcEvol] = EvolutionInversion(Phys,Num2);
        end
    
        % Initialise EVP structure
        [EVP2,Num2.Indx] = InitialiseEVP(Model,Phys,Num2);
        
        %%%
        
        % Run calculation
        Num2.OpLap1 = (Num2.OpD2 - CritWave^2*eye(Num2.Nz+1));
        Num2.OpLap2 = Num2.OpLap1*Num2.OpLap1;
        Num2.OpLapP = Num2.OpD1*diag(Phys.Perm)*Num2.OpD1 - diag(CritWave^2*Phys.Perm);
        
        [EVP2.OpMain,EVP2.OpGrow,EVP2.OpRayl] = MakeMainOperators(Model,Num2,CritWave);
        [Rp,PertVec] = EigValProbSolver(EVP2);
        
        % Get liquid fraction perturbation
        if Model.Medium == 2
            LiquidFracPert = [];%GetLiquidFractionPerturbation(Phys,Num,CritWave,PertVec);
        else
            LiquidFracPert = [];
        end
        
        % Assign output
        varargout = [varargout, [reshape(PertVec,Num.Nz+1,Model.NFields),LiquidFracPert] ];
        varargout = [varargout, Num.Grid];
    end
    

end

%% GetGrowthRates() Function

function [Growth] = GetGrowthRates(EVP,RaylVec)
% This function calculates the growth rates for a range of Rayleigh numbers
% with all other parameters held constant.
%
% Inputs: EVP   - The eigenvalue problem structure
%                   (See above for requirements)
%         RaylVec - Rayleigh number vector
%
% Output: Growth - The corresponding growth rates.
%
% Version 1.0
% (29/01/15)

    % Preparation
    Growth = NaN(length(RaylVec),1);

    % Loop over Rayleigh numbers
    for iR = 1:length(RaylVec)
        
        % Calculate growth rates
        Growth(iR) = EigValProbSolver(EVP,0,RaylVec(iR));
    end
end


%% Evolution inversion

function [ThermEvol,LqFrcEvol] = EvolutionInversion(Phys,Num)

    Vector = [Num.OpD2*Phys.Temp;zeros(Num.Nz+1,1)];
    
    Matrix = [
        diag( ones(Num.Nz+1,1) ) , diag(   Phys.Stefan*ones(Num.Nz+1,1) ) ;
        diag( Phys.LqdFrac     ) , diag( -(Phys.LqdusGrad+Phys.LqdusInf-Phys.Temp) ) ;
        ];
    
    Inversion = Matrix\Vector;
    ThermEvol = Inversion( (0*Num.Nz+1) : (1*Num.Nz+1) );
    LqFrcEvol = Inversion( (1*Num.Nz+2) : (2*Num.Nz+2) );
end


function LiquidFracPert = GetLiquidFractionPerturbation(Phys,Num,Wave,PertVec)

    LiquidFracPert = diag(Phys.ThermEvol)\[diag(-Phys.LqFrcEvol), Wave^2*Num.OpThermForc]*PertVec;
end
